@@ -1,3 +1,4 @@
+#!/usr/bin/python
 import os
 import subprocess
 import shutil
@@ -9,10 +10,20 @@ import dbus
 from dbus.mainloop.glib import DBusGMainLoop
 import time
 
+import options
+#from common.utils.options import get_start_timeout
+
 # Don't use /tmp (not enough space there)
 
-TEST_ENV_VARS =  { "XDG_DATA_HOME" : os.path.join (cfg.TEST_TMP_DIR, "xdg-data-home"),
-                   "XDG_CACHE_HOME": os.path.join (cfg.TEST_TMP_DIR, "xdg-cache-home")} 
+# Add this after fixing the backup/restore and ontology changes tests
+#"G_DEBUG" : "fatal_criticals",
+
+TEST_ENV_DIRS =  { "XDG_DATA_HOME" : os.path.join (cfg.TEST_TMP_DIR, "xdg-data-home"),
+                   "XDG_CACHE_HOME": os.path.join (cfg.TEST_TMP_DIR, "xdg-cache-home")}
+
+TEST_ENV_VARS = {  "TRACKER_DISABLE_MEEGOTOUCH_LOCALE": "",
+                   "LC_COLLATE": "en_GB.utf8"}
+
 EXTRA_DIRS = [os.path.join (cfg.TEST_TMP_DIR, "xdg-data-home", "tracker"),
               os.path.join (cfg.TEST_TMP_DIR, "xdg-cache-home", "tracker")]
 
@@ -28,7 +39,8 @@ class TrackerStoreLifeCycle ():
 
     def __init__ (self):
         self.timeout_id = 0
-
+        self.available = False
+        
     def start (self):
         """
         call this method to start and instance of tracker-store. It will return when the store is ready
@@ -48,7 +60,7 @@ class TrackerStoreLifeCycle ():
                                                          path="/org/freedesktop/DBus",
                                                          dbus_interface="org.freedesktop.DBus")
         self.store_proc = self.__start_tracker_store ()
-        
+
         # It should step out of this loop when the miner is visible in DBus
         self.loop.run ()
 
@@ -59,12 +71,13 @@ class TrackerStoreLifeCycle ():
         self.status_iface.Wait ()
         print "[store] ready."
 
-        
+
     def stop (self):
         self.__stop_tracker_store ()
-        # It should step out of this loop when the miner disappear from the bus
-        self.timeout_id = glib.timeout_add_seconds (REASONABLE_TIMEOUT, self.__timeout_on_idle)
-        self.loop.run ()
+        if self.available:
+            # It should step out of this loop when the miner disappear from the bus
+            self.timeout_id = glib.timeout_add_seconds (REASONABLE_TIMEOUT, self.__timeout_on_idle)
+            self.loop.run ()
 
         print "[store] stop."
         # Disconnect the signals of the next start we get duplicated messages
@@ -87,8 +100,10 @@ class TrackerStoreLifeCycle ():
         if name == cfg.TRACKER_BUSNAME:
             if old_owner == '' and new_owner != '':
                 print "[store] appears in the bus"
+                self.available = True
             elif old_owner != ''  and new_owner == '':
                 print "[store] disappears from the bus"
+                self.available = False
             else:
                 print "[store] name change %s -> %s" % (old_owner, new_owner)
             self.loop.quit ()
@@ -98,13 +113,22 @@ class TrackerStoreLifeCycle ():
         tracker = [tracker_binary]
         # The env variables can be passed as parameters!
         FNULL = open('/dev/null', 'w')
-        return subprocess.Popen (tracker, stdout=FNULL, stderr=FNULL)
+        if options.is_manual_start ():
+            print "Start tracker-store manually"
+        else:
+            return subprocess.Popen (tracker, stdout=FNULL, stderr=FNULL)
 
     def __stop_tracker_store (self):
         #control_binary = os.path.join (cfg.BINDIR, "tracker-control")
         #FNULL = open('/dev/null', 'w')
         #subprocess.call ([control_binary, "-t"], stdout=FNULL)
-        self.store_proc.terminate ()
+        if options.is_manual_start ():
+            if self.available:
+                print "Kill tracker-store manually"
+                # Quit when disappearing from the bus
+                self.loop.run ()
+        else:
+            self.store_proc.terminate ()
 
 
 class TrackerMinerFsLifeCycle():
@@ -113,7 +137,8 @@ class TrackerMinerFsLifeCycle():
     """
     def __init__ (self):
         self.timeout_id = 0
-    
+        self.available = False
+
     def start (self):
         """
         call this method to start and instance of miner-fs. It will return when the miner is 'Idle'
@@ -134,7 +159,7 @@ class TrackerMinerFsLifeCycle():
                                                          path="/org/freedesktop/DBus",
                                                          dbus_interface="org.freedesktop.DBus")
         self.__start_tracker_miner_fs ()
-        
+
         # It should step out of this loop when the miner is visible in DBus
         self.loop.run ()
 
@@ -146,21 +171,23 @@ class TrackerMinerFsLifeCycle():
         self.timeout_id = glib.timeout_add_seconds (REASONABLE_TIMEOUT, self.__timeout_on_idle)
         self.loop.run ()
 
-        
+
     def stop (self):
         self.__stop_tracker_miner_fs ()
         # It should step out of this loop when the miner disappear from the bus
-        self.timeout_id = glib.timeout_add_seconds (REASONABLE_TIMEOUT, self.__timeout_on_idle)
-        self.loop.run ()
+        if (self.available):
+            self.timeout_id = glib.timeout_add_seconds (REASONABLE_TIMEOUT, self.__timeout_on_idle)
+            self.loop.run ()
 
         # Disconnect the signals of the next start we get duplicated messages
         self.bus._clean_up_signal_match (self.name_owner_match)
         self.bus._clean_up_signal_match (self.status_match)
+        print "[miner-fs] stop."
 
 
     def wait_for_idle (self, timeout=REASONABLE_TIMEOUT):
         # The signal is already connected
-        print "[miner-fs] waiting for Idle"
+        print "\n[miner-fs] waiting for Idle"
         self.timeout_id = glib.timeout_add_seconds (timeout, self.__timeout_on_idle)
         self.loop.run ()
 
@@ -191,12 +218,19 @@ class TrackerMinerFsLifeCycle():
     def __start_tracker_miner_fs (self):
         miner_fs_binary = os.path.join (cfg.EXEC_PREFIX, "tracker-miner-fs")
         FNULL = open ('/dev/null', 'w')
-        return subprocess.Popen ([miner_fs_binary], stdout=FNULL, stderr=FNULL)
+        if options.is_manual_start ():
+            print "Start tracker-miner-fs manually"
+        else:
+            return subprocess.Popen ([miner_fs_binary], stdout=FNULL, stderr=FNULL)
 
     def __stop_tracker_miner_fs (self):
         control_binary = os.path.join (cfg.BINDIR, "tracker-control")
         FNULL = open('/dev/null', 'w')
-        subprocess.call ([control_binary, "-t"], stdout=FNULL)
+        if options.is_manual_start ():
+            print "Kill miner manually"
+            self.loop.run ()
+        else:
+            subprocess.call ([control_binary, "-t"], stdout=FNULL)
 
 
 class TrackerWritebackLifeCycle():
@@ -205,7 +239,7 @@ class TrackerWritebackLifeCycle():
     """
     def __init__ (self):
         self.timeout_id = 0
-    
+
     def start (self):
         """
         call this method to start and instance of writeback.
@@ -213,36 +247,44 @@ class TrackerWritebackLifeCycle():
         """
         self.loop = gobject.MainLoop()
         dbus_loop = DBusGMainLoop(set_as_default=True)
-        bus = dbus.SessionBus (dbus_loop)
+        self.bus = dbus.SessionBus (dbus_loop)
 
-        obj = bus.get_object ("org.freedesktop.DBus",
+        obj = self.bus.get_object ("org.freedesktop.DBus",
                               "/org/freedesktop/DBus")
         self.admin = dbus.Interface (obj, dbus_interface="org.freedesktop.DBus")
         if (self.admin.NameHasOwner (cfg.WRITEBACK_BUSNAME)):
             raise Exception ("Writeback is already running! kill it before starting this one")
 
-        bus.add_signal_receiver (self.__name_owner_changed_cb,
-                                 signal_name="NameOwnerChanged",
-                                 path="/org/freedesktop/DBus",
-                                 dbus_interface="org.freedesktop.DBus")
+        self.name_owner_match = self.bus.add_signal_receiver (self.__name_owner_changed_cb,
+                                                              signal_name="NameOwnerChanged",
+                                                              path="/org/freedesktop/DBus",
+                                                              dbus_interface="org.freedesktop.DBus")
         self.__start_tracker_writeback ()
 
         # It should step out of this loop when the writeback is visible in DBus
         self.timeout_id = glib.timeout_add_seconds (REASONABLE_TIMEOUT, self.__timeout_on_idle)
         self.loop.run ()
-        
+
     def stop (self):
-        assert self.process 
+        assert self.process
         self.process.kill ()
+        self.bus._clean_up_signal_match (self.name_owner_match)
+        print "[writeback] stop."
+        
 
     def __name_owner_changed_cb (self, name, old_owner, new_owner):
         if name == cfg.WRITEBACK_BUSNAME:
-            print "Writeback changed named '%s' -> '%s'" % (old_owner, new_owner)
+            if old_owner == '' and new_owner != '':
+                print "[writeback] appears in the bus"
+            elif old_owner != ''  and new_owner == '':
+                print "[writeback] disappears from the bus"
+            else:
+                print "[writeback] name change %s -> %s" % (old_owner, new_owner)
             if (self.timeout_id != 0):
                 glib.source_remove (self.timeout_id)
                 self.timeout_id = 0
             self.loop.quit ()
-            
+
     def __timeout_on_idle (self):
         print "Timeout... asumming idle"
         self.loop.quit ()
@@ -253,7 +295,10 @@ class TrackerWritebackLifeCycle():
         writeback = [writeback_binary]
         # The env variables can be passed as parameters!
         FNULL = open('/dev/null', 'w')
-        self.process = subprocess.Popen (writeback, stdout=FNULL, stderr=FNULL)
+        if options.is_manual_start ():
+            print "Start tracker-writeback manually"
+        else:
+            self.process = subprocess.Popen (writeback, stdout=FNULL, stderr=FNULL)
 
 
 class TrackerSystemAbstraction:
@@ -262,14 +307,14 @@ class TrackerSystemAbstraction:
         """
         Sets up the XDG_*_HOME variables and make sure the directories exist
         """
-        for var, directory in TEST_ENV_VARS.iteritems ():
+        for var, directory in TEST_ENV_DIRS.iteritems ():
             print "[Conf] Setting %s - %s" %(var, directory)
             self.__recreate_directory (directory)
             os.environ [var] = directory
 
         for directory in EXTRA_DIRS:
             self.__recreate_directory (directory)
-            
+
         if confdir :
             self.__recreate_directory (XDG_CONFIG_HOME_DIR)
             shutil.copytree (os.path.join (confdir, "tracker"),
@@ -282,6 +327,9 @@ class TrackerSystemAbstraction:
             print "[Conf] Setting %s - %s" % ("TRACKER_DB_ONTOLOGIES_DIR", ontodir)
             os.environ ["TRACKER_DB_ONTOLOGIES_DIR"] = ontodir
 
+        for var, value in TEST_ENV_VARS.iteritems ():
+            os.environ [var] = value
+
     def unset_up_environment (self):
         """
         Unset the XDG_*_HOME variables from the environment
@@ -289,7 +337,11 @@ class TrackerSystemAbstraction:
         for var, directory in TEST_ENV_VARS.iteritems ():
             if os.environ.has_key (var):
                 del os.environ [var]
-                
+
+        for var, directory in TEST_ENV_DIRS.iteritems ():
+            if os.environ.has_key (var):
+                del os.environ [var]
+
         if (os.environ.has_key ("XDG_CONFIG_HOME")):
             del os.environ ["XDG_CONFIG_HOME"]
 
@@ -303,13 +355,19 @@ class TrackerSystemAbstraction:
         """
         self.__stop_tracker_processes ()
         self.set_up_environment (confdir, ontodir)
-        
+
         self.store = TrackerStoreLifeCycle ()
         self.store.start ()
 
     def tracker_store_start (self):
         self.store.start ()
 
+    def tracker_store_stop_nicely (self):
+        self.store.stop ()
+
+    def tracker_store_stop_brutally (self):
+        self.store.kill ()
+
     def tracker_store_restart_with_new_ontologies (self, ontodir):
         self.store.stop ()
         if ontodir:
@@ -320,36 +378,32 @@ class TrackerSystemAbstraction:
         except dbus.DBusException, e:
             raise UnableToBootException ("Unable to boot the store \n(" + str(e) + ")")
 
-    def tracker_store_brutal_restart (self):
-        self.store.kill ()
-        self.store.start ()
-
     def tracker_store_prepare_journal_replay (self):
-        db_location = os.path.join (TEST_ENV_VARS ['XDG_CACHE_HOME'], "tracker", "meta.db")
+        db_location = os.path.join (TEST_ENV_DIRS ['XDG_CACHE_HOME'], "tracker", "meta.db")
         os.unlink (db_location)
 
-        lockfile = os.path.join (TEST_ENV_VARS ['XDG_DATA_HOME'], "tracker", "data", ".ismeta.running")
+        lockfile = os.path.join (TEST_ENV_DIRS ['XDG_DATA_HOME'], "tracker", "data", ".ismeta.running")
         f = open (lockfile, 'w')
         f.write (" ")
         f.close ()
 
     def tracker_store_corrupt_dbs (self):
-        db_path = os.path.join (TEST_ENV_VARS ['XDG_CACHE_HOME'], "tracker", "meta.db")
+        db_path = os.path.join (TEST_ENV_DIRS ['XDG_CACHE_HOME'], "tracker", "meta.db")
         f = open (db_path, "w")
         for i in range (0, 100):
             f.write ("Some stupid content... hohohoho, not a sqlite file anymore!\n")
         f.close ()
 
     def tracker_store_remove_journal (self):
-        db_location = os.path.join (TEST_ENV_VARS ['XDG_DATA_HOME'], "tracker", "data")
+        db_location = os.path.join (TEST_ENV_DIRS ['XDG_DATA_HOME'], "tracker", "data")
         shutil.rmtree (db_location)
         os.mkdir (db_location)
 
     def tracker_store_remove_dbs (self):
-        db_location = os.path.join (TEST_ENV_VARS ['XDG_CACHE_HOME'], "tracker")
+        db_location = os.path.join (TEST_ENV_DIRS ['XDG_CACHE_HOME'], "tracker")
         shutil.rmtree (db_location)
         os.mkdir (db_location)
-        
+
 
     def tracker_store_testing_stop (self):
         """
@@ -388,7 +442,7 @@ class TrackerSystemAbstraction:
         """
         self.miner_fs.stop ()
         self.store.stop ()
-        
+
         self.__stop_tracker_processes ()
         self.unset_up_environment ()
 
@@ -403,7 +457,15 @@ class TrackerSystemAbstraction:
         # Tracker write must have been started before
         self.writeback.stop ()
         self.tracker_miner_fs_testing_stop ()
-        
+
+    def tracker_all_testing_start (self, confdir=None):
+        # This will start all miner-fs, store and writeback
+        self.tracker_writeback_testing_start (confdir)
+
+    def tracker_all_testing_stop (self):
+        # This will stop all miner-fs, store and writeback
+        self.tracker_writeback_testing_stop ()
+
     #
     # Private API
     #
@@ -435,7 +497,7 @@ if __name__ == "__main__":
     print "   started, waiting 5 sec. to stop it"
     glib.timeout_add_seconds (5, destroy_the_world, a)
     gtk.main ()
-    
+
     print "-- Starting miner-fs --"
     b = TrackerMinerFsLifeCycle ()
     b.start ()
